@@ -16,7 +16,12 @@ from chester.zero_break.problem_specification import DataInfo
 class CategoricalPreModelAnalysis:
     def __init__(self, data_info: DataInfo):
         self.data_info = data_info
-        self.cols = self.data_info.feature_types_val["categorical"]
+        self.cols = list(set(self.data_info.feature_types_val["categorical"]))
+        # drop dups
+        if self.data_info.data.columns.duplicated().any():
+            drop_dup_data = self.data_info.data.loc[:, ~ self.data_info.data.columns.duplicated()]
+            self.data = drop_dup_data
+            self.data_info.data = drop_dup_data
         self.n_cols = len(self.cols)
         self.target = self.data_info.data[self.data_info.target]
         self.target_df = self.data_info.data[[self.data_info.target]]
@@ -26,21 +31,25 @@ class CategoricalPreModelAnalysis:
         self.cols_sorted_with_pvalue = None
 
     def tsne(self):
-        X_tsne_3d = TSNE(n_components=3).fit_transform(pd.get_dummies(self.data))
+        X = self.data.copy()
+        X = X.sample(n=min(5000, len(X)))
+        target = self.target[X.index]
+
+        X_tsne_3d = TSNE(n_components=3).fit_transform(pd.get_dummies(X))
         X_tsne_2d = X_tsne_3d[:, :2]
 
         fig = plt.figure(figsize=(16, 10))
         ax1 = plt
         # ax2 = fig.add_subplot(122, projection='3d')
         if self.data_info.problem_type_val in ["Regression"]:
-            ax1.hexbin(X_tsne_2d[:, 0], X_tsne_2d[:, 1], C=self.target, gridsize=50, cmap='viridis', edgecolors='black')
+            ax1.hexbin(X_tsne_2d[:, 0], X_tsne_2d[:, 1], C=target, gridsize=50, cmap='viridis', edgecolors='black')
             # ax2.scatter(X_tsne_3d[:, 0], X_tsne_3d[:, 1], X_tsne_3d[:, 2], c=self.target, cmap='viridis')
             ax1.title("Visualizing Categorical Features and Target with t-SNE (2D)")
             # ax2.set_title("Visualizing Categorical Features and Target with t-SNE (3D)")
         elif self.data_info.problem_type_val in ["Binary regression", "Binary classification"]:
-            target_classes = self.target.unique()
+            target_classes = target.unique()
             color_map = {target_class: color for target_class, color in zip(target_classes, ['red', 'blue'])}
-            colors = self.target.apply(lambda x: color_map[x])
+            colors = target.apply(lambda x: color_map[x])
             ax1.scatter(X_tsne_2d[:, 0], X_tsne_2d[:, 1], c=colors)
             # ax2.scatter(X_tsne_3d[:, 0], X_tsne_3d[:, 1], X_tsne_3d[:, 2], c=colors)
             legend_handles = [Patch(color=color_map[target_class], label=target_class) for target_class in
@@ -49,16 +58,15 @@ class CategoricalPreModelAnalysis:
             # ax2.set_title("Visualizing Categorical Features and Target with t-SNE (3D)")
             ax1.legend(handles=legend_handles)
         else:  # Multi-class classification
-            target_classes = self.target.unique()
+            target_classes = target.unique()
             color_map = {target_class: color for target_class, color in
                          zip(target_classes, plt.cm.rainbow(np.linspace(0, 1, len(target_classes))))}
             ax1.legend(
                 handles=[Patch(color=color_map[target_class], label=target_class) for target_class in target_classes])
             # ax2.legend(
             #     handles=[Patch(color=color_map[target_class], label=target_class) for target_class in target_classes])
-
-    def any_categorical(self):
-        return True if len(self.cols) > 0 else False
+        plt.show()
+        plt.close()
 
     def sort_by_pvalue(self):
         from sklearn.cluster import KMeans
@@ -90,7 +98,11 @@ class CategoricalPreModelAnalysis:
         return [x[0] for x in sorted_list]
 
     def analyze_pvalue(self, is_plot=True, top_features=10):
+        from chester.util import ReportCollector, REPORT_PATH
+        rc = ReportCollector(REPORT_PATH)
         self.sort_by_pvalue()
+        rc.save_object(self.cols_sorted_with_pvalue[0:50],
+                       text="top 50 with lowest partial pvalue for categorical feat based on chi square:")
         if len(self.cols) == 0:
             return None
         if is_plot:
@@ -117,44 +129,65 @@ class CategoricalPreModelAnalysis:
         feature_index = {feature: index for index, feature in enumerate(self.cols_sorted[0:top_features])}
         top_feature_names.sort(key=lambda x: feature_index[x])
         if self.data_info.problem_type_val in ["Binary regression", "Binary classification"]:
-            plt.close()
-            max_plots = min(9, top_features)
-            dim = math.ceil(math.sqrt(max_plots))
-            num_rows = math.ceil(max_plots / dim)
-            fig, ax = plt.subplots(num_rows, dim, figsize=(18, 15))
-            fig.tight_layout()
-            if classification_row_percent:
-                plt.suptitle("Partial Plot to Identify Patterns between Sampled Categorical Features and Target\n"
-                             "Showing % from Feature (row)",
-                             fontsize=14, fontweight='bold')
-            else:
-                plt.suptitle("Partial Plot to Identify Patterns between Sampled Categorical Features and Target\n"
-                             "Showing % from Target (column)",
-                             fontsize=14, fontweight='bold')
-            for i, col in enumerate(top_feature_names):
-                if i >= num_rows * num_rows:
-                    return None
-                if dim == 1:
-                    ax_i = ax
-                else:
-                    ax_i = ax[i // dim, i % dim]
+            if top_features == 1:
+                fig, ax = plt.subplots(figsize=(10, 8))
                 if classification_row_percent:
-                    crosstab = pd.crosstab(self.data[col], self.target, normalize='index') * 100
+                    plt.suptitle("Partial Plot to Identify Patterns between Sampled Categorical Features and Target\n"
+                                 "Showing % from Feature (row)",
+                                 fontsize=14, fontweight='bold')
+                else:
+                    plt.suptitle("Partial Plot to Identify Patterns between Sampled Categorical Features and Target\n"
+                                 "Showing % from Target (column)",
+                                 fontsize=14, fontweight='bold')
+                if classification_row_percent:
+                    crosstab = pd.crosstab(self.data[top_feature_names[0]], self.target, normalize='index') * 100
                     crosstab = crosstab[(crosstab.T != 0).any()]
                     crosstab = crosstab.loc[:, (crosstab != 0).any(axis=0)]
                     crosstab = crosstab.loc[crosstab.sum(axis=1).sort_values(ascending=False).index[:5]]
                 else:
-                    crosstab = pd.crosstab(self.data[col], self.target, normalize='columns') * 100
+                    crosstab = pd.crosstab(self.data[top_feature_names[0]], self.target, normalize='columns') * 100
                     crosstab = crosstab[(crosstab.T != 0).any()]
                     crosstab = crosstab.loc[:, (crosstab != 0).any(axis=0)]
                     crosstab = crosstab.loc[crosstab.sum(axis=1).sort_values(ascending=False).index[:5]]
-                sns.heatmap(crosstab, annot=False, cmap="YlGnBu", fmt='g', ax=ax_i)
-                ax_i.set_ylabel(None)
-                ax_i.set_xlabel(None)
-                ax_i.set_title(col, fontsize=12, fontweight='bold')
-            plt.tight_layout()
-            plt.show()
-            plt.close()
+                sns.heatmap(crosstab, annot=False, cmap="YlGnBu", fmt='g', ax=ax)
+                ax.set_ylabel(None)
+                ax.set_xlabel(None)
+                ax.set_title(top_feature_names[0], fontsize=12, fontweight='bold')
+                plt.show()
+                plt.close()
+
+            else:
+                max_plots = min(9, top_features)
+                dim = min(math.ceil(math.sqrt(max_plots)), 2)
+                fig, ax = plt.subplots(dim, dim, figsize=(18, 15))
+                fig.tight_layout()
+                if classification_row_percent:
+                    plt.suptitle("Partial Plot to Identify Patterns between Sampled Categorical Features and Target\n"
+                                 "Showing % from Feature (row)",
+                                 fontsize=14, fontweight='bold')
+                else:
+                    plt.suptitle("Partial Plot to Identify Patterns between Sampled Categorical Features and Target\n"
+                                 "Showing % from Target (column)",
+                                 fontsize=14, fontweight='bold')
+                for i, col in enumerate(top_feature_names):
+                    ax_i = ax[i // dim, i % dim]
+                    if classification_row_percent:
+                        crosstab = pd.crosstab(self.data[col], self.target, normalize='index') * 100
+                        crosstab = crosstab[(crosstab.T != 0).any()]
+                        crosstab = crosstab.loc[:, (crosstab != 0).any(axis=0)]
+                        crosstab = crosstab.loc[crosstab.sum(axis=1).sort_values(ascending=False).index[:5]]
+                    else:
+                        crosstab = pd.crosstab(self.data[col], self.target, normalize='columns') * 100
+                        crosstab = crosstab[(crosstab.T != 0).any()]
+                        crosstab = crosstab.loc[:, (crosstab != 0).any(axis=0)]
+                        crosstab = crosstab.loc[crosstab.sum(axis=1).sort_values(ascending=False).index[:5]]
+                    sns.heatmap(crosstab, annot=False, cmap="YlGnBu", fmt='g', ax=ax_i)
+                    ax_i.set_ylabel(None)
+                    ax_i.set_xlabel(None)
+                    ax_i.set_title(col, fontsize=12, fontweight='bold')
+                plt.tight_layout()
+                plt.show()
+                plt.close()
         if self.data_info.problem_type_val in ["Regression"]:
             max_plots = 9
             top_n = self.data[:top_features].columns
@@ -165,9 +198,6 @@ class CategoricalPreModelAnalysis:
 
             from sklearn.cluster import KMeans
             target = self.target
-            kmeans = KMeans(n_clusters=10)
-            kmeans.fit(self.target_df)
-            target_labels = kmeans.labels_
             plt.suptitle(
                 "Partial Plot to Identify Patterns between Categorical Sampled Features and Target (grouped by kmeans)",
                 fontsize=16,
@@ -181,12 +211,14 @@ class CategoricalPreModelAnalysis:
                 if column.dtype == "object":
                     column = column.astype("category").cat.codes
                     column = column[column < 5]
-                    column_name = self.data[col].astype("category").cat.categories[:5]
-                    data_filtered = self.target_df[column.isin(column_name)]
+                    top_values = list(self.data[col].astype("category").cat.categories[:5])
+                    data_filtered = self.data[self.data[col].isin(top_values)]
                     sns.boxplot(x=column, y=target, data=data_filtered)
                 plt.ylabel("Target")
                 plt.xlabel("{} Value".format(col))
                 plt.subplots_adjust(hspace=0.5, wspace=0.5)
+                plt.show()
+                plt.close()
         elif self.data_info.problem_type_val in ["Multiclass classification"]:
             max_plots = 16
             top_n = self.data[:top_features].columns
