@@ -1,24 +1,28 @@
+import random
 from collections import Counter
 
 import matplotlib
 import pandas as pd
 from chester.data_loader.webtext_data import load_data_pirates, load_data_king_arthur, load_data_chat_logs
 from flatbuffers.builder import np
-from sklearn.datasets import fetch_20newsgroups
+from sklearn.datasets import fetch_20newsgroups, fetch_openml
 
 from chester.run.full_run import run_madcat
-from chester.run.user_classes import Data, ModelRun
+from chester.run.user_classes import Data, ModelRun, TimeSeriesHandler
+import yfinance as yf
 
 matplotlib.use('TkAgg')
 target_column = 'target'
 
+
 ################################################################################################
-df1 = load_data_pirates().assign(target='pirate')  # .sample(300, replace=True)
-df2 = load_data_king_arthur().assign(target='arthur')  # .sample(300, replace=True)
-df3 = load_data_chat_logs().assign(target='chat')  # .sample(300, replace=True)
-df = pd.concat([df1, df2
-                   , df3
-                ])
+# df1 = load_data_pirates().assign(target='pirate').sample(300, replace=True)
+# df2 = load_data_king_arthur().assign(target='arthur').sample(300, replace=True)
+# df3 = load_data_chat_logs().assign(target='chat').sample(300, replace=True)
+# df = pd.concat([
+#     df1, df2,
+#     df3
+# ])
 
 # df['target'] = df['target'].apply(lambda x: 0 if "pirate" in x else 1)  # can do with or without
 ################################################################################################
@@ -41,13 +45,28 @@ df = pd.concat([df1, df2
 ###############################################################################################
 
 ###############################################################################################
-# df = pd.read_csv("chester/model_training/models/chester_models/day.csv")
+# df = pd.read_csv("/Users/amitosi/PycharmProjects/chester/chester/data/day.csv")
 # df.rename(columns={'cnt': 'target'}, inplace=True)
 ###############################################################################################
 
+###############################################################################################
+df = pd.read_csv("/Users/amitosi/PycharmProjects/chester/chester/data/daily_cinema.csv")
+df.rename(columns={'humidity': 'target'}, inplace=True)
+###############################################################################################
+
+###############################################################################################
+# df = pd.read_csv("chester/model_training/models/chester_models/day.csv")
+# df = pd.read_csv("/Users/amitosi/PycharmProjects/chester/chester/data/time_series_kaggle.csv")
+# df = df[['date', 'store_nbr', 'family', 'sales']]
+#
+# top_stores = df.groupby('store_nbr').size().sort_values(ascending=False)[:1000].index
+# df = df[df['store_nbr'].isin(top_stores)]
+# df.rename(columns={'sales': 'target'}, inplace=True)
+###############################################################################################
+
+
 ################################################################################################
 # from sklearn import datasets
-#
 # digits = datasets.load_digits()
 # X = digits.images.reshape((len(digits.images), -1))
 # df = pd.DataFrame(X)
@@ -119,10 +138,6 @@ df = pd.concat([df1, df2
 # df1 = pd.read_csv("chester/run_manual_chester/loans_1.csv")
 # df = df1.copy(deep=False)
 # df.rename(columns={'Loan_Status': 'target'}, inplace=True)
-# feature_types = {
-#     'numeric': ['ApplicantIncome', 'CoapplicantIncome', 'LoanAmount', 'Credit_History'],
-#     'boolean': [], 'text': [],
-#     'categorical': ['Gender', 'Married', 'Dependents', 'Education', 'Self_Employed', 'property_Area'], 'time': []}
 # df.dropna(subset=['target'], inplace=True)
 ###############################################################################################
 
@@ -154,18 +169,11 @@ def generate_data(n_features, n_rows, target_type='binary'):
     return df
 
 
-# df = generate_data(20, 1000, target_type='binary')
+# df = generate_data(1, 100, target_type='binary')
 # df = generate_data(30, 100, target_type='multiclass')
 ###############################################################################################
 
 
-def load_ex1():
-    data = pd.read_csv("chester/model_training/models/chester_models/lead_df_2023-01-23.csv")
-    data.rename(columns={'lawer_conversion_label': target_column}, inplace=True)
-    # data.drop(columns=["LEAD_ID"], inplace=True)
-    return data
-
-
 def load_ex2():
     newsgroups_train = fetch_20newsgroups(subset='train')
     df = pd.DataFrame(newsgroups_train.data, columns=['text'])
@@ -205,20 +213,49 @@ def load_ex5():
 
 
 # load data
-# df = load_vlad()
-# df = load_ex1()
 # df = load_ex2()
 # df = load_ex3().sample(1000)
 # df = load_ex4().sample(1000)
 # df = load_ex5().sample(900)
 
-#
-madcat_collector = run_madcat(Data(df=df, target_column='target'),
+
+def load_yaho(tickers=None, start_date='2010-01-01', end_date='2023-02-15'):
+    # Download stock data for all tickers and concatenate them into a single dataframe
+    if tickers is None:
+        tickers = ['AAPL', 'MSFT', 'GOOG']
+    df_list = []
+    for ticker in tickers:
+        ticker_df = yf.download(ticker, start=start_date, end=end_date)
+        ticker_df["id"] = ticker
+        df_list.append(ticker_df[['Close', 'id']])
+
+    # Combine all dataframes into one and reset the index
+    df = pd.concat(df_list).reset_index()
+
+    # Rename the columns to match the desired output
+    df = df.rename(columns={'Date': 'date', 'Close': 'value'})
+
+    # Rearrange the columns to match the desired output
+    df = df[['id', 'date', 'value']]
+    df.rename(columns={'value': 'target'}, inplace=True)
+
+    return df
+
+
+df = load_yaho()
+print("df shape", df.shape)
+print("df cols", df.columns)
+
+madcat_collector = run_madcat(Data(df=df.sample(100), target_column='target'),
                               is_feature_stats=True,
+                              # time_series_handler=TimeSeriesHandler(id_cols=['store_nbr', 'family']),
+                              # time_series_handler=TimeSeriesHandler(id_cols=["id"]),
+                              time_series_handler=TimeSeriesHandler(),
                               is_pre_model=True,
                               is_model_training=True,
-                              model_run=ModelRun(n_models=1),
-                              is_post_model=True, is_model_weaknesses=True
+                              model_run=ModelRun(n_models=2),
+                              is_post_model=True, is_model_weaknesses=True,
+                              plot=True
                               )
 
 # output_collector = run_madcat(
